#include "DOMErrorBinding.h"
#include "ErrorEventBinding.h"
#include "EventBinding.h"
#include "EventTargetBinding.h"
#include "MessageEventBinding.h"
#include "MessagePortBinding.h"
#include "PromiseBinding.h"
#include "PromiseDebuggingBinding.h"
#include "TextDecoderBinding.h"
#include "TextEncoderBinding.h"
#include "URLSearchParamsBinding.h"
#include "WorkerBinding.h"
#include "js/Id.h"
#include "mozilla/dom/BindingUtils.h"
#include "nsThreadUtils.h"

namespace mozilla {
namespace dom {
bool
ResolveSystemBinding(JSContext* aCx, JS::Handle<JSObject*> aObj, JS::Handle<jsid> aId, bool* aResolvedp)
{
  MOZ_ASSERT(NS_IsMainThread());

  static jsid sChromeWorker_id;
  static jsid sDOMError_id;
  static jsid sErrorEvent_id;
  static jsid sEvent_id;
  static jsid sEventTarget_id;
  static jsid sMessageEvent_id;
  static jsid sMessagePort_id;
  static jsid sPromise_id;
  static jsid sPromiseDebugging_id;
  static jsid sTextDecoder_id;
  static jsid sTextEncoder_id;
  static jsid sURLSearchParams_id;
  static jsid sWorker_id;

  static bool idsInited = false;
  if (!idsInited) {
    if (!InternJSString(aCx, sChromeWorker_id, "ChromeWorker")) {
      return false;
    }

    if (!InternJSString(aCx, sDOMError_id, "DOMError")) {
      return false;
    }

    if (!InternJSString(aCx, sErrorEvent_id, "ErrorEvent")) {
      return false;
    }

    if (!InternJSString(aCx, sEvent_id, "Event")) {
      return false;
    }

    if (!InternJSString(aCx, sEventTarget_id, "EventTarget")) {
      return false;
    }

    if (!InternJSString(aCx, sMessageEvent_id, "MessageEvent")) {
      return false;
    }

    if (!InternJSString(aCx, sMessagePort_id, "MessagePort")) {
      return false;
    }

    if (!InternJSString(aCx, sPromise_id, "Promise")) {
      return false;
    }

    if (!InternJSString(aCx, sPromiseDebugging_id, "PromiseDebugging")) {
      return false;
    }

    if (!InternJSString(aCx, sTextDecoder_id, "TextDecoder")) {
      return false;
    }

    if (!InternJSString(aCx, sTextEncoder_id, "TextEncoder")) {
      return false;
    }

    if (!InternJSString(aCx, sURLSearchParams_id, "URLSearchParams")) {
      return false;
    }

    if (!InternJSString(aCx, sWorker_id, "Worker")) {
      return false;
    }

    idsInited = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sChromeWorker_id) && ChromeWorkerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!ChromeWorkerBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sDOMError_id) {
    if (!DOMErrorBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sErrorEvent_id) {
    if (!ErrorEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sEvent_id) {
    if (!EventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sEventTarget_id) {
    if (!EventTargetBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sMessageEvent_id) {
    if (!MessageEventBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sMessagePort_id) {
    if (!MessagePortBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sPromise_id) {
    if (!PromiseBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sPromiseDebugging_id) && PromiseDebuggingBinding::ConstructorEnabled(aCx, aObj)) {
    if (!PromiseDebuggingBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sTextDecoder_id) {
    if (!TextDecoderBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sTextEncoder_id) {
    if (!TextEncoderBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if (JSID_IS_VOID(aId) || aId == sURLSearchParams_id) {
    if (!URLSearchParamsBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  if ((JSID_IS_VOID(aId) || aId == sWorker_id) && WorkerBinding::ConstructorEnabled(aCx, aObj)) {
    if (!WorkerBinding::GetConstructorObject(aCx, aObj)) {
      return false;
    }
    *aResolvedp = true;
  }

  return true;
}

} // namespace dom
} // namespace mozilla

